package sample;

import java.io.Reader;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.ibatis.io.Resources;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;

import sample.biz.domain.Owner;
import sample.biz.domain.Pet;
import sample.dao.PetMapper;

public class MyBatis3Main {

	public static void main(String[] args) throws Exception {
		String resource = "sample/mybatis.MyBatis3Main.xml";
		Reader reader = Resources.getResourceAsReader(resource);
		SqlSessionFactory sqlMapper = new SqlSessionFactoryBuilder().build(reader);
		SqlSession session = sqlMapper.openSession();
		
		// 파라미터가 하나인 바인딩
		Pet pet = session.selectOne("sample.dao.PetMapper.selectPet", 1);
		System.out.println(pet.getPetName());
		
		// JavaBeans를 사용한 파라미터의 바인딩과 키 자동생성
		Pet newPet = new Pet();
		newPet.setPetName("나비");
		newPet.setOwnerName("홍길동");
		newPet.setPrice(3000);
		newPet.setBirthDate(new Date());
		session.insert("sample.dao.PetMapper.insertPet", newPet);
		
		// 복수의 JavaBeans 바인딩 
		Map<String, Object> map = new HashMap<String, Object>();
		pet = new Pet();
		pet.setPetName("나비");
		map.put("pet", pet);
		Owner owner = new Owner();
		owner.setOwnerName("홍길동");
		map.put("owner", owner);
		Pet result = session.selectOne("sample.dao.PetMapper.selectPetByNames", map);
		System.out.println(pet.getPetName());

		// resultMap을 사용한 복잡한 매핑
		owner = session.selectOne("sample.dao.PetMapper.selectOwner", "홍길동");
		for (Pet p: owner.getPetList()) {
			System.out.println(p.getPetName());
		}
		
		// 동적 SQL
		pet = new Pet();
		pet.setPrice(1000);
		List<Pet> list = session.selectList("sample.dao.PetMapper.findPet", pet);
		for (Pet p : list) {
			System.out.println(p.getPetName());
		}
		
		// Mapper 인터페이스 
		PetMapper mapper = session.getMapper(PetMapper.class);
		pet = mapper.selectPet(1);
		System.out.println(pet.getPetName());
		
		
		
		session.delete("sample.dao.PetMapper.deletePet", "나비");
		
		session.commit();
		session.close();
	}
	
}
